@@ -13,6 +13,7 @@
 --------------------------
 
 .. automodule:: spykeutils.correlations
+    :members:
 
 :mod:`progress_indicator` Module
 --------------------------------
@@ -27,7 +28,28 @@
 
 .. automodule:: spykeutils.rate_estimation
     :members:
-    :exclude-members: binned_spike_trains, psth, spike_density_estimation
+
+:mod:`signal_processing` Module
+-------------------------------
+
+.. automodule:: spykeutils.signal_processing
+    :members:
+    :show-inheritance:
+    :undoc-members:
+
+:mod:`spike_train_generation` Module
+------------------------------------
+
+.. automodule:: spykeutils.spike_train_generation
+    :members:
+    :undoc-members:
+
+:mod:`spike_train_metrics` Module
+------------------------------------
+
+.. automodule:: spykeutils.spike_train_metrics
+    :members:
+    :undoc-members:
 
 :mod:`sorting_quality_assesment` Module
 ---------------------------------------
@@ -42,10 +64,16 @@
 
 .. automodule:: spykeutils.stationarity
     :members:
-    :exclude-members: spike_amplitude_histogram
+
+:mod:`tools` Module
+------------------------
+
+.. automodule:: spykeutils.tools
+    :members:
 """
 
-__version__ = '0.2.1'
+__version__ = '0.3.0'
+
 
 class SpykeException(Exception):
     """ Exception thrown when a function in spykeutils encounters a
@@ -56,4 +84,4 @@ class SpykeException(Exception):
         (and therefore be visible in the console) for easier
         debugging.
     """
-    pass
\ No newline at end of file
+    pass
