@@ -1,6 +1,3 @@
-"""
-.. autofunction:: signals(signals, events=None, epochs=None, spike_trains=None, spikes=None, use_subplots=True, time_unit=s, y_unit=None, progress=None)
-"""
 from __future__ import division
 
 import scipy as sp
@@ -19,8 +16,8 @@ import helper
 
 @helper.needs_qt
 def signals(signals, events=None, epochs=None, spike_trains=None,
-           spikes=None, show_waveforms=True, use_subplots=True,
-           time_unit=pq.s, y_unit=None, progress=None):
+            spikes=None, show_waveforms=True, use_subplots=True,
+            subplot_names=True, time_unit=pq.s, y_unit=None, progress=None):
     """ Create a plot from a list of analog signals.
 
     :param list signals: The list of :class:`neo.core.AnalogSignal` objects
@@ -40,6 +37,8 @@ def signals(signals, events=None, epochs=None, spike_trains=None,
         shown as waveforms (if available) or vertical lines.
     :param bool use_subplots: Determines if a separate subplot for is created
         each signal.
+    :param bool subplot_names: Only valid if ``use_subplots`` is True.
+        Determines if signal (or channel) names are shown for subplots.
     :param Quantity time_unit: The unit of the x axis.
     :param progress: Set this parameter to report progress.
     :type progress: :class:`spykeutils.progress_indicator.ProgressIndicator`
@@ -85,6 +84,19 @@ def signals(signals, events=None, epochs=None, spike_trains=None,
 
     channels = range(len(signals))
 
+    channel_indices = []
+    for s in signals:
+        if not s.recordingchannel:
+            channel_indices.append(-1)
+        else:
+            channel_indices.append(s.recordingchannel.index)
+
+    # Heuristic: If multiple channels have the same index, use channel order
+    # as index for spike waveforms
+    nonindices = max(0, channel_indices.count(-1) - 1)
+    if len(set(channel_indices)) != len(channel_indices) - nonindices:
+        channel_indices = range(len(signals))
+
     progress.set_ticks((len(spike_trains) + len(spikes) + 1) * len(channels))
 
     offset = 0 * signals[0].units
@@ -94,11 +106,13 @@ def signals(signals, events=None, epochs=None, spike_trains=None,
             pW = BaseCurveWidget(win)
             plot = pW.plot
 
-            if signals[c].name:
-                win.set_plot_title(plot, signals[c].name)
-            elif signals[c].recordingchannel:
-                if signals[c].recordingchannel.name:
-                    win.set_plot_title(plot, signals[c].recordingchannel.name)
+            if subplot_names:
+                if signals[c].name:
+                    win.set_plot_title(plot, signals[c].name)
+                elif signals[c].recordingchannel:
+                    if signals[c].recordingchannel.name:
+                        win.set_plot_title(
+                            plot, signals[c].recordingchannel.name)
 
             sample = (1 / signals[c].sampling_rate).simplified
             x = (sp.arange(signals[c].shape[0])) * sample + signals[c].t_start
@@ -111,7 +125,8 @@ def signals(signals, events=None, epochs=None, spike_trains=None,
                 plot.add_item(make.curve(x, signals[c]))
             helper.add_events(plot, events, x.units)
 
-            _add_spike_waveforms(plot, spikes, x.units, c, offset, progress)
+            _add_spike_waveforms(
+                plot, spikes, x.units, channel_indices[c], offset, progress)
 
             for train in spike_trains:
                 color = helper.get_object_color(train.unit)
@@ -119,8 +134,8 @@ def signals(signals, events=None, epochs=None, spike_trains=None,
                 progress.step()
 
             win.add_plot_widget(pW, c)
-            plot.set_axis_unit(BasePlot.Y_LEFT,
-                signals[c].dimensionality.string)
+            plot.set_axis_unit(
+                BasePlot.Y_LEFT, signals[c].dimensionality.string)
             progress.step()
 
         plot.set_axis_title(BasePlot.X_BOTTOM, 'Time')
@@ -148,11 +163,12 @@ def signals(signals, events=None, epochs=None, spike_trains=None,
             x.units = time_unit
 
             if y_unit is not None:
-                plot.add_item(make.curve(x,
-                    (signals[c] + offset).rescale(y_unit)))
+                plot.add_item(
+                    make.curve(x, (signals[c] + offset).rescale(y_unit)))
             else:
                 plot.add_item(make.curve(x, signals[c] + offset))
-            _add_spike_waveforms(plot, spikes, x.units, c, offset, progress)
+            _add_spike_waveforms(
+                plot, spikes, x.units, channel_indices[c], offset, progress)
             offset += max_offset
             progress.step()
 
@@ -188,7 +204,9 @@ def signals(signals, events=None, epochs=None, spike_trains=None,
 
 def _add_spike_waveforms(plot, spikes, x_units, channel, offset, progress):
     for spike in spikes:
-        if spike.waveform is None:
+        if spike.waveform is None or not spike.sampling_rate:
+            continue
+        if channel < 0 or spike.waveform.shape[1] <= channel:
             continue
 
         color = helper.get_object_color(spike.unit)
@@ -196,13 +214,14 @@ def _add_spike_waveforms(plot, spikes, x_units, channel, offset, progress):
             lsweep = spike.left_sweep
         else:
             lsweep = 0.0 * pq.ms
-        start = (spike.time-lsweep).rescale(x_units)
+        start = (spike.time - lsweep).rescale(x_units)
         stop = (spike.waveform.shape[0] / spike.sampling_rate +
                 spike.time - lsweep).rescale(x_units)
-        spike_x = sp.arange(start, stop,
+        spike_x = sp.arange(
+            start, stop,
             (1.0 / spike.sampling_rate).rescale(x_units)) * x_units
 
-        plot.add_item(make.curve(spike_x,
-            spike.waveform[:, channel] + offset,
-            color=color, linewidth=2))
+        plot.add_item(
+            make.curve(spike_x, spike.waveform[:, channel] + offset,
+                       color=color, linewidth=2))
         progress.step()
\ No newline at end of file
