@@ -2,6 +2,7 @@ import os
 import sys
 from copy import copy
 from collections import OrderedDict
+import traceback
 
 import neo
 
@@ -36,6 +37,8 @@ class NeoDataProvider(DataProvider):
         io, blocks = cls._load_neo_file(filename, lazy)
         if io and hasattr(io, 'close'):
             io.close()
+        if blocks is None:
+            return None
         return blocks[index]
 
     @classmethod
@@ -52,21 +55,37 @@ class NeoDataProvider(DataProvider):
     @classmethod
     def _load_neo_file(cls, filename, lazy):
         """ Returns a NEO io object and a list of contained blocks for a
-            file name. This function also caches all loaded blocks
-            :Parameters:
-                filename : str
-                    The full path of the file (relative or absolute)
-                lazy : bool
-                    Determines if lazy mode is used for NEO io
+        file name. This function also caches all loaded blocks
+
+        :param str filename:
+            The full path of the file (relative or absolute).
+        :param bool lazy:
+            Determines if lazy mode is used for NEO io.
         """
         if os.path.isdir(filename):
             for io in neo.io.iolist:
                 if io.mode == 'dir':
-                    n_io = io(filename)
-                    block = n_io.read(lazy=lazy)
-                    cls.block_indices[block] = 0
-                    cls.loaded_blocks[filename] = [block]
-                    return n_io, [block]
+                    try:
+                        n_io = io(filename)
+                        block = n_io.read(lazy=lazy)
+                        if io == neo.TdtIO and not block.segments:
+                            # TdtIO can produce empty blocks for invalid dirs
+                            continue
+
+                        cls.block_indices[block] = 0
+                        cls.loaded_blocks[filename] = [block]
+                        return n_io, [block]
+                    except Exception, e:
+                        sys.stderr.write(
+                            'Load error for directory "%s":\n' % filename)
+                        tb = sys.exc_info()[2]
+                        while not ('self' in tb.tb_frame.f_locals and
+                                   tb.tb_frame.f_locals['self'] == n_io):
+                            if tb.tb_next is not None:
+                                tb = tb.tb_next
+                            else:
+                                break
+                        traceback.print_exception(type(e), e, tb)
         else:
             extension = filename.split('.')[-1]
             for io in neo.io.iolist:
@@ -75,22 +94,36 @@ class NeoDataProvider(DataProvider):
                         # Fix unicode problem with pyinstaller
                         if hasattr(sys, 'frozen'):
                             filename = filename.encode('UTF-8')
-                        n_io = io(filename=filename)
-                        blocks = n_io.read_all_blocks(lazy=lazy)
+
+                    n_io = io(filename=filename)
+
+                    try:
+                        if hasattr(io, 'read_all_blocks'):  # Neo 0.2.1
+                            blocks = n_io.read_all_blocks(lazy=lazy)
+                        else:
+                            content = n_io.read(lazy=lazy)
+                            if isinstance(content, neo.Block):  # Neo 0.2.1
+                                cls.block_indices[content] = 0
+                                cls.loaded_blocks[filename] = [content]
+                                return n_io, [content]
+                            blocks = content
+                        # Neo >= 0.3.0, read() returns a list of blocks
                         for i, b in enumerate(blocks):
                             cls.block_indices[b] = i
                         cls.loaded_blocks[filename] = blocks
                         return n_io, blocks
-                    try:
-                        n_io = io(filename=filename)
-                        block = n_io.read(lazy=lazy)
-                        cls.block_indices[block] = 0
-                        cls.loaded_blocks[filename] = [block]
-                        return n_io, [block]
-                    except Exception:
-                        sys.stderr.write('Load error with '+str(io)+
-                                         ' for file '+filename+'\n')
-                        continue
+                    except Exception, e:
+                        sys.stderr.write(
+                            'Load error for file "%s":\n' % filename)
+                        tb = sys.exc_info()[2]
+                        while not ('self' in tb.tb_frame.f_locals and
+                                   tb.tb_frame.f_locals['self'] == n_io):
+                            if tb.tb_next is not None:
+                                tb = tb.tb_next
+                            else:
+                                break
+                        traceback.print_exception(type(e), e, tb)
+
         return None, None
 
     @staticmethod
@@ -150,10 +183,10 @@ class NeoDataProvider(DataProvider):
         selected_units = viewer.neo_units()
         for u in selected_units:
             segment_indices[u] = len(segment_list)
-            rcg_id = None if u.recordingchannelgroup is None\
-            else u.recordingchannelgroup.units.index(u)
-            rcg = rcg_indices[u.recordingchannelgroup]\
-            if u.recordingchannelgroup else None
+            rcg_id = None if u.recordingchannelgroup is None \
+                else u.recordingchannelgroup.units.index(u)
+            rcg = rcg_indices[u.recordingchannelgroup] \
+                if u.recordingchannelgroup else None
             unit_list.append([rcg_id, rcg])
         data['units'] = unit_list
 
@@ -174,13 +207,13 @@ class NeoDataProvider(DataProvider):
                 segment = copy(s)
                 segment.analogsignals = [sig for sig in s.analogsignals
                                          if sig.recordingchannel
-                in selected_channels]
-                segment.analogsignalarrays =\
-                [asa for asa in s.analogsignalarrays
-                 if asa.recordingchannelgroup in selected_rcgs]
-                segment.irregularlysampledsignals =\
-                [iss for iss in s.irregularlysampledsignals
-                 if iss.recordingchannel in selected_channels]
+                                         in selected_channels]
+                segment.analogsignalarrays = \
+                    [asa for asa in s.analogsignalarrays
+                     if asa.recordingchannelgroup in selected_rcgs]
+                segment.irregularlysampledsignals = \
+                    [iss for iss in s.irregularlysampledsignals
+                     if iss.recordingchannel in selected_channels]
                 segment.spikes = [sp for sp in s.spikes
                                   if sp.unit in selected_units]
                 segment.spiketrains = [st for st in s.spiketrains
@@ -192,9 +225,9 @@ class NeoDataProvider(DataProvider):
         for old_rcg in old.recordingchannelgroups:
             if old_rcg in selected_rcgs:
                 rcg = copy(old_rcg)
-                rcg.analogsignalarrays =\
-                [asa for asa in old_rcg.analogsignalarrays
-                 if asa.segment in selected_segments]
+                rcg.analogsignalarrays = \
+                    [asa for asa in old_rcg.analogsignalarrays
+                     if asa.segment in selected_segments]
 
                 rcg.recordingchannels = []
                 for c in old_rcg.recordingchannels:
@@ -203,10 +236,10 @@ class NeoDataProvider(DataProvider):
                     channel = copy(c)
                     channel.analogsignals = [sig for sig in c.analogsignals
                                              if sig.segment
-                    in selected_segments]
-                    channel.irregularlysampledsignals =\
-                    [iss for iss in c.irregularlysampledsignals
-                     if iss.segment in selected_segments]
+                                             in selected_segments]
+                    channel.irregularlysampledsignals = \
+                        [iss for iss in c.irregularlysampledsignals
+                         if iss.segment in selected_segments]
                     channel.recordingchannelgroups = copy(
                         c.recordingchannelgroups)
                     channel.recordingchannelgroups.insert(
@@ -244,7 +277,7 @@ class NeoDataProvider(DataProvider):
         units = self.units()
         for s in self.segments():
             trains.extend([t for t in s.spiketrains if t.unit in units or
-                                                       t.unit is None])
+                           t.unit is None])
         for u in self.units():
             trains.extend([t for t in u.spiketrains if t.segment is None])
 
@@ -258,7 +291,7 @@ class NeoDataProvider(DataProvider):
         segments = self.segments()
         for u in self.units():
             st = [t for t in u.spiketrains if t.segment in segments or
-                                                  t.segment is None]
+                  t.segment is None]
             if st:
                 trains[u] = st
 
@@ -278,7 +311,7 @@ class NeoDataProvider(DataProvider):
         units = self.units()
         for s in self.segments():
             st = [t for t in s.spiketrains if t.unit in units or
-                                              t.unit is None]
+                  t.unit is None]
             if st:
                 trains[s] = st
 
@@ -326,7 +359,7 @@ class NeoDataProvider(DataProvider):
         units = self.units()
         for s in self.segments():
             spikes.extend([t for t in s.spikes if t.unit in units or
-                                                  t.unit is None])
+                           t.unit is None])
         for u in self.units():
             spikes.extend([t for t in u.spikes if t.segment is None])
 
@@ -340,7 +373,7 @@ class NeoDataProvider(DataProvider):
         segments = self.segments()
         for u in self.units():
             sp = [t for t in u.spikes if t.segment in segments or
-                                         t.segment is None]
+                  t.segment is None]
             if sp:
                 spikes[u] = sp
 
@@ -360,7 +393,7 @@ class NeoDataProvider(DataProvider):
         units = self.units()
         for s in self.segments():
             sp = [t for t in s.spikes if t.unit in units or
-                                         t.unit is None]
+                  t.unit is None]
             if sp:
                 spikes[s] = sp
 
@@ -401,7 +434,7 @@ class NeoDataProvider(DataProvider):
 
         return spikes
 
-    def events(self, include_array_events = True):
+    def events(self, include_array_events=True):
         """ Return a dictionary (indexed by Segment) of lists of
         Event objects.
         """
@@ -416,7 +449,7 @@ class NeoDataProvider(DataProvider):
                     ret[s].extend(convert.event_array_to_events(a))
         return ret
 
-    def labeled_events(self, label, include_array_events = True):
+    def labeled_events(self, label, include_array_events=True):
         """ Return a dictionary (indexed by Segment) of lists of Event
         objects with the given label.
         """
@@ -443,7 +476,7 @@ class NeoDataProvider(DataProvider):
                 ret[s] = s.eventarrays
         return ret
 
-    def epochs(self, include_array_epochs = True):
+    def epochs(self, include_array_epochs=True):
         """ Return a dictionary (indexed by Segment) of lists of
         Epoch objects.
         """
@@ -458,7 +491,7 @@ class NeoDataProvider(DataProvider):
                     ret[s].extend(convert.epoch_array_to_epochs(a))
         return ret
 
-    def labeled_epochs(self, label, include_array_epochs = True):
+    def labeled_epochs(self, label, include_array_epochs=True):
         """ Return a dictionary (indexed by Segment) of lists of Epoch
         objects with the given label.
         """
@@ -485,46 +518,44 @@ class NeoDataProvider(DataProvider):
                 ret[s] = s.epocharrays
         return ret
 
-    def num_analog_signals(self, mode=1):
-        """ Return the number of :class:`neo.core.AnalogSignal` objects.
-        """
-        return len(self.analog_signals(mode))
-
-    def analog_signals(self, mode=1):
+    def analog_signals(self, conversion_mode=1):
         """ Return a list of :class:`neo.core.AnalogSignal` objects.
         """
         signals = []
         channels = self.recording_channels()
 
-        if mode == 1 or mode == 3:
+        if conversion_mode == 1 or conversion_mode == 3:
             for s in self.segments():
                 signals.extend([t for t in s.analogsignals
-                               if t.recordingchannel in channels or
-                                  t.recordingchannel is None])
+                                if t.recordingchannel in channels or
+                                t.recordingchannel is None])
             for u in self.recording_channels():
                 signals.extend([t for t in u.analogsignals
                                 if t.segment is None])
-        if mode > 1:
+        if conversion_mode > 1:
             for sa in self.analog_signal_arrays():
                 for sig in convert.analog_signal_array_to_analog_signals(sa):
-                    if sig.recordingchannel is None or \
-                        sig.recordingchannel in channels:
+                    if (sig.recordingchannel is None or
+                            sig.recordingchannel in channels):
                         signals.append(sig)
 
         return signals
 
-    def analog_signals_by_segment(self, mode=1):
+    def analog_signals_by_segment(self, conversion_mode=1):
         """ Return a dictionary (indexed by Segment) of lists of
         :class:`neo.core.AnalogSignal` objects.
         """
         signals = OrderedDict()
         channels = self.recording_channels()
 
-        if mode == 1 or mode == 3:
+        if conversion_mode == 1 or conversion_mode == 3:
             for s in self.segments():
-                sig = [t for t in s.analogsignals
-                       if t.recordingchannel in channels or
-                          t.recordingchannel is None]
+                sig = []
+                for c in channels:
+                    sig.extend([t for t in c.analogsignals
+                                if t.segment == s])
+                sig.extend([t for t in s.analogsignals
+                            if t.recordingchannel is None])
                 if sig:
                     signals[s] = sig
 
@@ -535,33 +566,33 @@ class NeoDataProvider(DataProvider):
             if nonesignals:
                 signals[self.no_segment] = nonesignals
 
-        if mode > 1:
+        if conversion_mode > 1:
             for o, sa_list in \
-                self.analog_signal_arrays_by_segment().iteritems():
+                    self.analog_signal_arrays_by_segment().iteritems():
                 for sa in sa_list:
                     for sig in \
-                        convert.analog_signal_array_to_analog_signals(sa):
+                            convert.analog_signal_array_to_analog_signals(sa):
                         if sig.recordingchannel is None or \
-                            sig.recordingchannel in channels:
+                                sig.recordingchannel in channels:
                             if o not in signals:
                                 signals[o] = []
                             signals[o].append(sig)
 
         return signals
 
-    def analog_signals_by_channel(self, mode=1):
+    def analog_signals_by_channel(self, conversion_mode=1):
         """ Return a dictionary (indexed by RecordingChannel) of lists
         of :class:`neo.core.AnalogSignal` objects.
         """
         signals = OrderedDict()
         channels = self.recording_channels()
 
-        if mode == 1 or mode == 3:
+        if conversion_mode == 1 or conversion_mode == 3:
             segments = self.segments()
             for c in channels:
                 sig = [t for t in c.analogsignals
                        if t.segment in segments or
-                          t.segment is None]
+                       t.segment is None]
                 if sig:
                     signals[c] = sig
 
@@ -572,12 +603,12 @@ class NeoDataProvider(DataProvider):
             if nonesignals:
                 signals[self.no_channel] = nonesignals
 
-        if mode > 1:
+        if conversion_mode > 1:
             for o, sa_list in \
-                self.analog_signal_arrays_by_channelgroup().iteritems():
+                    self.analog_signal_arrays_by_channelgroup().iteritems():
                 for sa in sa_list:
                     for sig in \
-                        convert.analog_signal_array_to_analog_signals(sa):
+                            convert.analog_signal_array_to_analog_signals(sa):
                         if sig.recordingchannel is None:
                             if self.no_channel not in signals:
                                 signals[self.no_channel] = [sig]
@@ -591,7 +622,7 @@ class NeoDataProvider(DataProvider):
 
         return signals
 
-    def analog_signals_by_channel_and_segment(self, mode=1):
+    def analog_signals_by_channel_and_segment(self, conversion_mode=1):
         """ Return a dictionary (indexed by RecordingChannel) of
         dictionaries (indexed by Segment) of :class:`neo.core.AnalogSignal`
         lists.
@@ -599,7 +630,7 @@ class NeoDataProvider(DataProvider):
         signals = OrderedDict()
         channels = self.recording_channels()
 
-        if mode == 1 or mode == 3:
+        if conversion_mode == 1 or conversion_mode == 3:
             segments = self.segments()
             for c in channels:
                 for s in segments:
@@ -623,12 +654,12 @@ class NeoDataProvider(DataProvider):
             if nonesignals:
                 signals[self.no_channel] = nonesignals
 
-        if mode > 1:
-            for cg, inner in self.analog_signal_arrays_by_channelgroup_and_segment().iteritems():
+        if conversion_mode > 1:
+            sigs = self.analog_signal_arrays_by_channelgroup_and_segment()
+            for cg, inner in sigs.iteritems():
                 for seg, sa_list in inner.iteritems():
                     for sa in sa_list:
-                        for sig in \
-                            convert.analog_signal_array_to_analog_signals(sa):
+                        for sig in convert.analog_signal_array_to_analog_signals(sa):
                             chan = sig.recordingchannel
                             if chan not in channels:
                                 continue
@@ -638,14 +669,8 @@ class NeoDataProvider(DataProvider):
                                 signals[chan][seg] = []
                             signals[chan][seg].append(sig)
 
-
         return signals
 
-    def num_analog_signal_arrays(self):
-        """ Return the number of AnalogSignalArray objects.
-        """
-        return len(self.analog_signal_arrays())
-
     def analog_signal_arrays(self):
         """ Return a list of :class:`neo.core.AnalogSignalArray` objects.
         """
@@ -668,8 +693,12 @@ class NeoDataProvider(DataProvider):
         signals = OrderedDict()
         channelgroups = self.recording_channel_groups()
         for s in self.segments():
-            sa = [t for t in s.analogsignalarrays
-                  if t.recordingchannelgroup in channelgroups]
+            sa = []
+            for c in channelgroups:
+                sa.extend([t for t in c.analogsignalarrays
+                           if t.segment == s])
+            sa.extend([t for t in s.analogsignalarrays
+                       if t.recordingchannelgroup is None])
             if sa:
                 signals[s] = sa
 
